using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using InfiniTec.Exchange;

namespace ExampleApplication
{
    [View("Connection properties", "Settings", AddAutomatically=false)]
    public partial class ConnectionView: UserControl
    {
        static Connection _Connection;

        static public Connection Connection
        {
            get
            {
                return _Connection;
            }
        }

        private void UpdateConnection()
        {
            if (_Connection == null) _Connection = new Connection();

            _Connection.ConnectionProtocol =
                (chkUseSsl.Checked) ? ConnectionProtocol.Secure : ConnectionProtocol.Default;

            _Connection.ExchangeServer = tbServer.Text;
            _Connection.Credential.Username = tbUsername.Text;
            _Connection.Credential.Password = tbPassword.Text;
            _Connection.Credential.AuthenticationType = (AuthenticationType) Enum.Parse(
                typeof(AuthenticationType), cbAuthenticationTypes.SelectedItem.ToString());

            if (rbUseExchangeServer.Checked)
            {
                _Connection.ActiveDirectoryServer = tbServer.Text;
            }
            else if (rbServerlessBinding.Checked)
            {
                _Connection.ActiveDirectoryServer = null;
                _Connection.UseGlobalCatalog = chkUseGC.Checked;
            }
            else if (rbSpecificServer.Checked)
            {
                _Connection.ActiveDirectoryServer = tbADServer.Text;
            }
        }

        public ConnectionView()
        {
            InitializeComponent();
        }

    
        private void ConnectionView_Load(object sender, EventArgs e)
        {
            if (cbAuthenticationTypes.Items.Count == 0) FillAuthenticationType();
        }

        private void FillAuthenticationType()
        {
            foreach (string s in Enum.GetNames(typeof(AuthenticationType)))
            {
                cbAuthenticationTypes.Items.Add(s);
            }
            cbAuthenticationTypes.SelectedIndex = 0;
        }

        private void chkUseSsl_CheckedChanged(object sender, EventArgs e)
        {
            UpdateConnection();
        }

        private void cbAuthenticationTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateConnection();
        }

        private void tbServer_Validated(object sender, EventArgs e)
        {
            UpdateConnection();
        }

        private void rbADServerSelectionChanged(object sender, EventArgs e)
        {
            chkUseGC.Enabled = rbServerlessBinding.Checked;
            tbADServer.Enabled = rbSpecificServer.Checked;

            UpdateConnection();
        }

        private void tbADServer_Validated(object sender, EventArgs e)
        {
            UpdateConnection();
        }

        private void chkUseGC_CheckedChanged(object sender, EventArgs e)
        {
            UpdateConnection();
        }
    }
}
