using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using InfiniTec.Exchange;
using InfiniTec.Threading;
using AsyncOperation=InfiniTec.Threading.AsyncOperation;

namespace ExampleApplication.Examples.Active_Directory_Integration_Examples
{
    [View("Enumerate Address Lists", "Active Directory integration")]
    public partial class EnumerateAddressLists : UserControl
    {
        public EnumerateAddressLists()
        {
            InitializeComponent();
        }

        private void btnFindAddressLists_Click(object sender, EventArgs e)
        {
            Connection connection;

            try
            {
                Cursor.Current = Cursors.WaitCursor;

                connection = ConnectionView.Connection;

                AdressListManager manager = new AdressListManager(connection);

                manager.Refresh();

                PopulateAddressLists(manager.GlobalAddressLists, "lvgGlobalAddressLists");
                PopulateAddressLists(manager.AllAddressLists, "lvgAllAddressLists");
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void PopulateAddressLists(IDictionary<string, AddressList> lists, string groupName)
        {
            foreach (AddressList list in lists.Values)
            {
                ListViewItem item;

                item = new ListViewItem(list.Name, lvAddressLists.Groups[groupName]);
                item.Tag = list;

                lvAddressLists.Items.Add(item);
            }
        }

        private void btnFindMembers_Click(object sender, EventArgs e)
        {
            AddressList list;
            Dictionary<User, ListViewItem> users;

            list = (AddressList) lvAddressLists.SelectedItems[0].Tag;

            list.FindMembers();

            lvMembers.Items.Clear();
            users = new Dictionary<User, ListViewItem>();

            foreach (IAddressListEntry entry in list.Members)
            {
                ListViewItem item;

                item = new ListViewItem(entry.Name);

                if (entry is User)
                {
                    item.SubItems.Add("User");
                    users.Add((User) entry, item);
                }
                else if (entry is Contact)
                {
                    item.SubItems.Add("Contact");
                }

                item.Tag = entry;

                lvMembers.Items.Add(item);
            }

            ResolveMailboxAdressesAsync(users);
        }

        private void ResolveMailboxAdressesAsync(Dictionary<User, ListViewItem> users)
        {
            AsyncOperation op;

            op = new AsyncOperation(ResolveMailboxAdressesInternal(users));

            op.Error += ResolveMailbox_Error;

            op.Run();
        }

        static void ResolveMailbox_Error(object sender, ExceptionEventArgs e)
        {
            MessageBox.Show("Could not resolve mail address. Exception: " + e.Exception);
        }

        private IEnumerable<OperationAction> ResolveMailboxAdressesInternal(Dictionary<User, ListViewItem> users)
        {
            User user;

            foreach (KeyValuePair<User, ListViewItem> item in users)
            {
                user = item.Key;

                using (AsyncOperation op = user.GetRefreshOperation())
                {
                    yield return new NestedOperation(op, false);
                }

                item.Value.SubItems.Add(user.Mailbox.Url.ToString());
            }
        }

        private void lvMembers_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
    }
}
