using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using InfiniTec.DirectoryServices;
using InfiniTec.Exchange;
using InfiniTec.Exchange.ActiveDirectory;
using InfiniTec.Exchange.WebDav;
using InfiniTec.Exchange.WebDav.Request;
using InfiniTec.Threading;

namespace ExampleApplication.Examples
{
    [View("Resolve user to mailbox", "Active Directory integration")]
    public partial class ResolveMailbox : UserControl
    {
        public ResolveMailbox()
        {
            InitializeComponent();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            PrincipalResolver resolver;
            InfiniTec.Exchange.Connection connection;

            connection = ConnectionView.Connection;

            resolver = new PrincipalResolver("", connection.ActiveDirectoryConnection);
            resolver.FindCompleted += new EventHandler<AsyncCompletedEventArgs>(resolver_FindCompleted);
            resolver.Filter = tbUsername.Text;
            resolver.ResolveTypes = ResolveTypes.User;

            Log("Resolving " + tbUsername.Text);
    
            resolver.FindAsync();
        }

        void resolver_FindCompleted(object sender, AsyncCompletedEventArgs e)
        {
            PrincipalResolver resolver;
            User user;

            Log("Resolve completed");

            resolver = (PrincipalResolver) sender;

            if (e.Error != null)
            {
                ViewManager.ShowException(e.Error);
                return;
            }

            if (resolver.SearchResult.Count == 0)
            {
                Log("Found nothing. Aborting.");
                return;
            }
            if (resolver.SearchResult.Count > 1)
            {
                Log("Found more than one user... Please specify a valid user");
                return;
            }

            user = new User((ActiveDirectoryUser) resolver.SearchResult[0], ConnectionView.Connection);

            user.RefreshCompleted += new AsyncCompletedEventHandler(user_RefreshCompleted);

            user.RefreshAsync();
        }

        void user_RefreshCompleted(object sender, AsyncCompletedEventArgs e)
        {
            User user;
            Mailbox mailbox;

            Log("Refresh completed");

            user = (User) sender;

            if (e.Error != null)
            {
                Log("Refresh failed: " + e.Error.Message);
                return;
            }

            mailbox = user.Mailbox;
            Log("Mailbox uri is: " + mailbox.Url);
            Log("Refreshing mailbox...");
            mailbox.RefreshCompleted += new AsyncCompletedEventHandler(mailbox_RefreshCompleted);
            mailbox.RefreshProgressChanged += new ProgressChangedEventHandler<RequestStatus>(mailbox_RefreshProgressChanged);

            mailbox.RefreshAsync();
        }

        void mailbox_RefreshProgressChanged(object sender, ProgressChangedEventArgs<RequestStatus> e)
        {
            Log(e.Status.ToString());
        }

        void mailbox_RefreshCompleted(object sender, AsyncCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                ViewManager.ShowException(e.Error);
            }
            else
            {
                Log("Refresh completed");
                Log("Path to mailbox is: " + ((Mailbox) sender).Url);
            }
        }

        private void Log(string entry)
        {
            lbLog.Items.Add(entry);
        }
    }
}

