using System;
using System.Collections.Generic;
using System.Drawing;
using System.Data;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using InfiniTec.Exchange;
using InfiniTec.Threading;

namespace ExampleApplication.Examples.Active_Directory_Integration_Examples
{
    [View("Find User objects", "Active Directory integration")]
    public partial class ResolveUsers : UserControl
    {
        public ResolveUsers()
        {
            InitializeComponent();
        }

        private void btnFind_Click(object sender, EventArgs e)
        {
            UserResolver resolver;
            Dictionary<User, ListViewItem> users;

            resolver = new UserResolver(ConnectionView.Connection);

            resolver.Filter = tbFilter.Text;

            resolver.Resolve();

            lvMembers.Items.Clear();
            users = new Dictionary<User, ListViewItem>();

            foreach (User entry in resolver.SearchResult)
            {
                ListViewItem item;

                item = new ListViewItem(!string.IsNullOrEmpty(entry.DisplayName) ? entry.DisplayName: entry.NT4AccountName);
                item.Tag = entry;

                users.Add(entry, item);


                lvMembers.Items.Add(item);
            }

            ResolveMailboxAdressesAsync(users);
        }

        private static void ResolveMailboxAdressesAsync(Dictionary<User, ListViewItem> users)
        {
            AsyncOperation op;

            op = new AsyncOperation(ResolveMailboxAdressesInternal(users));

            op.Run();
        }

        private static IEnumerable<OperationAction> ResolveMailboxAdressesInternal(Dictionary<User, ListViewItem> users)
        {
            User user;

            foreach (KeyValuePair<User, ListViewItem> item in users)
            {
                user = item.Key;

                using (AsyncOperation op = user.GetRefreshOperation())
                {
                    yield return new NestedOperation(op, false, typeof(Exception));
                    switch (op.Status) 
                    {
                        case OperationStatus.Error: 
                            item.Value.SubItems.Add(op.Exception.Message);
                            break;
                        case OperationStatus.Completed: 
                            item.Value.SubItems.Add(user.Mailbox.Url.ToString());
                            break;
                    }
                }

                
            }
        }
    }
}
