using System;
using System.Security.Principal;
using System.Windows.Forms;
using ExampleApplication.Example;
using InfiniTec.Exchange.Security;
using InfiniTec.Exchange.WebDav;
using InfiniTec.Security;

namespace ExampleApplication.Examples.Low_Level_Examples
{
    [View("Update Item security descriptor", "Security Examples")]
    public partial class UpdateItemSecurityDescriptor : ExampleBase
    {
        SimpleItem _Item;
        ItemSecurity _Security;

        public UpdateItemSecurityDescriptor()
        {
            InitializeComponent();
        }

        private void UpdateSecurityDescriptor_Load(object sender, EventArgs e)
        {
            foreach (string entry in Enum.GetNames(typeof(AccessMasks)))
            {
                lvAccessMasks.Items.Add(entry);
            }
        }

        private void lvAccessMasks_SelectedIndexChanged(object sender, EventArgs e)
        {
            IdentityResolver id;
            ItemAce ace;

            if (listView1.SelectedItems.Count == 0) return;

            ace = listView1.SelectedItems[0].Tag as ItemAce;

            if (ace == null) return;

            id = new IdentityResolver(ace.IdentityReference, ConnectionView.Connection.ExchangeServer);
            tbName.Text = id.Account.Value;
            //cbRoles.SelectedValue = ace.Role;

            DisplaySecurityMask(ace.AccessMask);
        }

        private void DisplaySecurityMask(AccessMasks accessMask)
        {
            AccessMasks mask;

            foreach (ListViewItem item in lvAccessMasks.Items)
            {
                mask = (AccessMasks)Enum.Parse(typeof(AccessMasks), item.Text);
                item.Checked = (accessMask & mask) == mask;
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            _Item = new SimpleItem(RootPath, ConnectionView.Connection);
            _Item.Refresh();

            _Security = _Item.GetSecurity();

            UpdateListview();
        }

        private void UpdateListview()
        {
            listView1.Items.Clear();
            foreach (ItemAce ace in _Security.GetAccessControlEntries())
            {
                AddAce(ace);
            }
        }

        private void AddAce(ItemAce entry)
        {
            ListViewItem item;
            IdentityResolver account;

            account = new IdentityResolver(entry.IdentityReference, ConnectionView.Connection.ExchangeServer);
            item = new ListViewItem();
            item.Text = account.Account.Value;
            item.SubItems.Add(account.AccountType.ToString());
            item.Tag = entry;

            item.SubItems.Add(entry.AccessMask.ToString());

            listView1.Items.Add(item);
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            IdentityResolver identity;
            AccessMasks mask;

            identity = new IdentityResolver(new NTAccount(tbName.Text),
                ConnectionView.Connection.ExchangeServer);

            mask = AccessMasks.None; 

            foreach (ListViewItem item in lvAccessMasks.Items)
            {
                if (!item.Checked) continue;

                mask |= (AccessMasks)Enum.Parse(typeof(AccessMasks), item.Text);
            }

            _Security.SetAccess(identity.Sid, mask);

       
            UpdateListview();
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 0) return;

            ItemAce ace = listView1.SelectedItems[0].Tag as ItemAce;
            _Security.RemoveAccess(ace.IdentityReference);
            listView1.Items.Remove(listView1.SelectedItems[0]);
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            _Item.SetSecurity(_Security);
            _Item.Save();
        }
      
    }
}

