using System;
using System.Collections.Generic;
using System.DirectoryServices.Protocols;
using System.Globalization;
using System.Net;
using System.Text;
using InfiniTec.DirectoryServices;
using InfiniTec.Exchange.Properties;
using InfiniTec.Threading;

namespace InfiniTec.Exchange.ActiveDirectory
{
    class ExchangeServer
    {
        private Item _ServerNode;
        private Capabilities _Capabilities;
        
        internal ExchangeServer(Item serverNode)
        {
            int versionNumber;

            _ServerNode = serverNode;
            versionNumber =
                serverNode.Properties.GetProperty<int>(WellknownProperties.ExchangeServer.VersionNumber).Value;
            if (versionNumber >= 6944) _Capabilities |= Capabilities.WebDavReplication;
        }    

        /// <summary>
       /// Returns the TCP/IP address for the this server object
        /// </summary>
        public string TcpIpAddress
        {
            get
            {
                foreach (string address in _ServerNode.Properties.GetMultivaluedProperty<string>(
                        WellknownProperties.ExchangeServer.NetworkAddress).Values)
                {
                    if (address.StartsWith("ncacn_ip_tcp:", StringComparison.OrdinalIgnoreCase))
                    {
                        return address.Substring(13);
                    }
                }
                throw new ServerTcpIPAddressNotFoundException(
                    string.Format(CultureInfo.CurrentCulture, Resources.ServerTcpIpAddressNotFoundException, _ServerNode.DistinguishedName));
            }
        }
    }


}
