using System;
using System.Collections.Generic;
using System.DirectoryServices.Protocols;
using System.Text;
using InfiniTec.DirectoryServices;
using InfiniTec.Exchange.Properties;
using InfiniTec.Threading;

namespace InfiniTec.Exchange.ActiveDirectory
{
    class ExchangeServerManager
    {
        private DirectoryServices.Connection _Connection;


        public ExchangeServerManager(DirectoryServices.Connection connection)
        {
            _Connection = connection;
        }

        public AsyncOperation<ExchangeServer> GetExchangeServer(string ldapPath)
        {
            return new AsyncOperation<ExchangeServer>(GetExchangeServerInternal(ldapPath, _Connection));
        }

        private static IEnumerable<OperationAction> GetExchangeServerInternal(string ldapPath, InfiniTec.DirectoryServices.Connection connection)
        {
            Item item;

            item = new Item(ldapPath, connection);
            item.PropertiesToLoad.Add(WellknownProperties.ExchangeServer.NetworkAddress);
            item.PropertiesToLoad.Add(WellknownProperties.ExchangeServer.VersionNumber);

            using (AsyncOperation operation = item.GetRefreshOperation())
            {
                yield return new NestedOperation(operation, false, typeof(DirectoryOperationException));
                if (operation.Status == OperationStatus.Error)
                {
                    DirectoryOperationException ex;

                    ex = (DirectoryOperationException)operation.Exception;

                    if (ex.Response.ResultCode == ResultCode.NoSuchObject)
                    {
                        throw new MailboxUrlResolutionFailedException(Resources.MailboxUrlResolutionFailedException, ex);
                    }
                    else throw ex;
                }

                yield return new OperationResult<ExchangeServer>(
                    new ExchangeServer(item));
            }
        }
    }
}
