using System.Collections.Generic;
using System.IO;
using System.Text;
using InfiniTec.DirectoryServices;
using InfiniTec.Threading;

namespace InfiniTec.Exchange
{
    class ActiveDirectoryReference: EntryIdReference
    {
        private string _LegacyExchangeDN;

        protected override IEnumerable<OperationAction>  ResolveInternal(Connection connection)
        {
            PrincipalResolver resolver;
            ActiveDirectoryPrincipal entry;

            resolver = new PrincipalResolver("", connection.ActiveDirectoryConnection);
            resolver.Filter = string.Format("{0}={1}", ActiveDirectory.WellknownProperties.User.LegacyExchangeDN,
                                            _LegacyExchangeDN);
            resolver.FilterType = FilterType.RawLdapFilter;

            resolver.ResolveTypes = ResolveTypes.Groups | ResolveTypes.User | ResolveTypes.Contacts;
            resolver.PropertiesToLoad.Add(ActiveDirectory.WellknownProperties.User.HomeMDB);
            resolver.PropertiesToLoad.Add(ActiveDirectory.WellknownProperties.User.ProxyAddresses);
            resolver.SearchMode = DirectoryServices.SearchMode.FindAll;

            yield return new NestedOperation(resolver.GetFindOperation(), false);

            if (resolver.SearchResult.Count == 0) yield return new OperationResult<IAddressListEntry>(null);

            entry = resolver.SearchResult[0];

            if (entry is ActiveDirectoryUser)
            {
                yield return new OperationResult<IAddressListEntry>(new User((ActiveDirectoryUser) entry, connection));
            }
            else if (entry is ActiveDirectoryContact)
            {
                yield return new OperationResult<IAddressListEntry>(new Contact((ActiveDirectoryContact) entry));
            }
            else
            {
                yield return new OperationResult<IAddressListEntry>(new Group((ActiveDirectoryGroup) entry));
            }
        }

        public ActiveDirectoryReference(BinaryReader reader)
        {
            string name;

            reader.ReadUInt32();

            reader.ReadUInt32();

            name = Encoding.ASCII.GetString(
                reader.ReadBytes((int) (reader.BaseStream.Length - reader.BaseStream.Position)));
            
            if (name.EndsWith("\0"))
            {
                _LegacyExchangeDN = name.Substring(0, name.Length - 1);
            }
            else
            {
                _LegacyExchangeDN = name;
            }
        }
    }
}
