using System.Collections.ObjectModel;
using System.Diagnostics;
using InfiniTec.Exchange.WebDav;

namespace InfiniTec.Exchange
{
    internal class ChangeNotifyDictionary<TKey, TValue> : KeyedCollection<TKey, TValue>, IKeyedCollection<TKey, TValue>
        where TValue : ICollectionEntry<TKey>
    {
        private Item _Item;

        public Item Item
        {
            [DebuggerStepThrough]
            get { return _Item; }
            [DebuggerStepThrough]
            set { _Item = value; }
        }

        protected override TKey GetKeyForItem(TValue item)
        {
            return item.GetKey();
        }

        public void Save()
        {
            foreach (TValue item in Items)
            {
                item.Save();
            }
        }

        protected override void ClearItems()
        {
            foreach (TValue item in Items)
            {
                item.Delete();
            }
            base.ClearItems();
        }

        protected override void InsertItem(int index, TValue item)
        {
            item.SetItem(_Item);

            base.InsertItem(index, item);
        }

        protected override void SetItem(int index, TValue item)
        {
            item.SetItem(_Item);

            base.SetItem(index, item);
        }

        protected override void RemoveItem(int index)
        {
            Items[index].Delete();

            base.RemoveItem(index);
        }

        public ChangeNotifyDictionary(Item item)
        {
            _Item = item;
        }

        TValue IKeyedCollection<TKey, TValue>.this[TKey index]
        {
            get { return this[index]; }
        }
    }
}