using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using InfiniTec.Exchange.WebDav;
using InfiniTec.Threading;

namespace InfiniTec.Exchange
{
    abstract class EntryIdReference
    {
        private static readonly Guid ExchangeAddressBook = new Guid("c840a7dc-42c0-1a10-b4b9-08002b2fe182");
        private static readonly Guid OutlookAddressBook = new Guid("0aaa42fe-c718-101a-e885-0b651c240000");
        private static readonly Guid OneOffEntryId = new Guid("a41f2b81-a3be-1910-9d6e-00dd010f5402");
        private static readonly Guid WindowsAddressBook = new Guid("d3ad91c0-9d51-11cf-a4a9-00aa0047faa4");

        public AsyncOperation<IAddressListEntry> GetResolveOperation(Connection connection)
        {
            return new AsyncOperation<IAddressListEntry>(ResolveInternal(connection));
        }

        protected abstract IEnumerable<OperationAction> ResolveInternal(Connection connection);

        internal static EntryIdReference DecodeEntryId(byte[] buffer, Guid localStoreId, string baseUrl)
        {
            BinaryReader reader;

            using (reader = new BinaryReader(new MemoryStream(buffer)))
            {
                return DecodeEntry(reader, localStoreId, baseUrl);
            }
        }

        private static EntryIdReference DecodeEntry(BinaryReader reader, Guid localStoreId, string baseUrl)
        {
            Guid id;

            reader.ReadUInt32();
            id = new Guid(reader.ReadBytes(16));

            if (id == WindowsAddressBook)
            {
                return DecodeWindowsAddressBookEntry(reader, localStoreId, baseUrl);
            }
            else if (id == ExchangeAddressBook)
            {
                return DecodeActiveDirectoryEntry(reader);
            }
            else if (id == OutlookAddressBook)
            {
                return new ExchangeStoreReference(reader, localStoreId, baseUrl);
            }
            else if (id== OneOffEntryId)
            {
                return new OneOffReference(reader);
            }

            throw new NotImplementedException();
        }

        private static EntryIdReference DecodeWindowsAddressBookEntry(BinaryReader reader, Guid localStoreId, string baseUrl)
        {
            WindowsAddressBookEntryType entryType;

            entryType = (WindowsAddressBookEntryType)reader.ReadByte();

            switch (entryType & WindowsAddressBookEntryType.ObjectTypeMask)
            {
                case WindowsAddressBookEntryType.None:
                case WindowsAddressBookEntryType.DirectoryPerson:
                case WindowsAddressBookEntryType.DirectoryDL:
                    return DecodeEntry(reader, localStoreId, baseUrl);
                case WindowsAddressBookEntryType.ContactPerson:
                case WindowsAddressBookEntryType.ContactDL:
                    return DecodeExchangeAddressBookEntry(reader, entryType, localStoreId, baseUrl);
                default:
                    throw new NotImplementedException();
            }
        }

        private static EntryIdReference DecodeActiveDirectoryEntry(BinaryReader reader)
        {
            return new ActiveDirectoryReference(reader);
        }

        private static EntryIdReference DecodeExchangeAddressBookEntry(BinaryReader reader, WindowsAddressBookEntryType entryType, Guid localStoreId, string baseUrl)
        {
            return new ExchangeStoreReference(reader, entryType, localStoreId, baseUrl);
        }
    }
}
