using System;
using System.Globalization;
using System.Net;
using InfiniTec.Exchange.Properties;

namespace InfiniTec.Exchange
{
    /// <summary>
    /// This class contains information about a HTTP status result
    /// </summary>
    public class HttpStatus
    {
        private HttpStatusCode _StatusCode;
        private string _Status;

        /// <summary>
        /// Returns the description of the status
        /// </summary>
        public string Status
        {
            get { return _Status; }
        }

        /// <summary>
        /// Returns the <see cref="HttpStatusCode"/> of the instance.
        /// </summary>
        public HttpStatusCode StatusCode
        {
            get { return _StatusCode; }
        }

        /// <summary>
        /// Creates a new instance from the specified status string.
        /// </summary>
        /// <param name="status">A string containing the status information</param>
        public HttpStatus(string status)
        {
            string[] parts;

            if (string.IsNullOrEmpty(status)) throw new ArgumentNullException("status");
            parts = status.Split(new char[] {' '}, 3);

            _StatusCode = (HttpStatusCode) int.Parse(parts[1], CultureInfo.InvariantCulture);
            _Status = parts[2];
        }

        /// <summary>
        /// Returns the <see cref="StatusCode"/> of the instance.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return StatusCode.ToString();
        }

        internal Exception GetException()
        {
            switch (StatusCode)
            {
                case HttpStatusCode.Unauthorized:
                    return new UnauthorizedAccessException(Resources.AccessDenied);
                case HttpStatusCode.NotFound:
                    return new PropertyNotFoundException(Resources.PropertyNotFound);
                default:
                    return new InvalidOperationException(string.Format(CultureInfo.CurrentCulture, Resources.InvalidStatusCode, StatusCode));
            }
        }
    }
}