using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using InfiniTec.Threading;

namespace InfiniTec.Exchange
{
    class OneOffReference: EntryIdReference
    {
        private const uint UnicodeEncoding = 0x80000000;

        private EmailAddress _Address;

        public OneOffReference(BinaryReader reader)
        {
            uint flags;
            Encoding encoding;
            string displayName;
            string addressType;
            string address;

            flags = reader.ReadUInt32();
            encoding = ((flags & UnicodeEncoding) == UnicodeEncoding) ? Encoding.Unicode : Encoding.ASCII;

            displayName = ReadString(reader, encoding);
            addressType = ReadString(reader, encoding);
            address = ReadString(reader, encoding);
            _Address = new EmailAddress(displayName, address, addressType);
        }

        private static string ReadString(BinaryReader reader, Encoding encoding)
        {
            byte[] buffer;
            char[] chars;
            Stream stream;
            int index;
            string result;
            int streamPosition;

            stream = reader.BaseStream;
            streamPosition = (int) stream.Position;

            buffer = new byte[stream.Length - streamPosition];

            stream.Read(buffer, 0, buffer.Length);

            chars = encoding.GetChars(buffer);

            index = 0;
            while (index < chars.Length && chars[index] != '\0') index++;

            result = new string(chars, 0, index);

            streamPosition += encoding.GetByteCount(result + "\0");
            stream.Position = streamPosition;

            return result;
        }

        protected override IEnumerable<OperationAction> ResolveInternal(Connection connection)
        {
            yield return new OperationResult<IAddressListEntry>(_Address);
        }
    }
}
