namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class represents a public or private folder on an Exchange store.
    /// </summary>
    public class Folder<ItemType> : FolderBase
        where ItemType : Item, new()
    {
        private ItemCollection<ItemType> _ChildItems;

        /// <summary>
        /// Initializes a new instance of the <see cref="Folder&lt;ItemType&gt;"/> class.
        /// </summary>
        /// <param name="path">The path of the folder on the server.</param>
        /// <param name="connection">The <see cref="Connection"/> to use.</param>
        /// <param name="propertiesToLoad">The properties to retrieve. If omitted, all properties are retrieved.</param>
        public Folder(string path, Connection connection, params string[] propertiesToLoad)
            : base(path, connection, propertiesToLoad)
        {
            Initialize();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Folder&lt;ItemType&gt;"/> class.
        /// </summary>
        /// <param name="path">The path.</param>
        /// <param name="connection">The connection.</param>
        /// <param name="isNew">if set to <c>true</c> [is new].</param>
        /// <param name="propertiesToLoad">The properties to load.</param>
        protected Folder(string path, Connection connection, bool isNew, params string[] propertiesToLoad)
            : base(path, connection, isNew, propertiesToLoad)
        {
            Initialize();
        }

        private void Initialize()
        {
            _ChildItems = new ItemCollection<ItemType>(Url, Connection);
        }

        /// <summary>
        /// Returns an ItemCollection which provides access to the child items of the current instance.
        /// </summary>
        /// <value>An ItemCollection which provides access to the child items of the current instance.</value>
        public ItemCollection<ItemType> ChildItems
        {
            get
            {
                return _ChildItems;
            }
        }
       
    }
}